function [t,s_old,sig_new, f,spec_old,spec_new] = ampl_k(s,sample_rate,b)
//  Realizes amplitude filter. Removes frequencies with frequency-spectrum-amplitude 
//  lower than b-times of the biggest freq-spectrum element.
//  
//  Calling sequence  
//  [t,s_old,sig_new, f,spec_old,spec_new] = ampl_k(s,sample_rate,b)
//    
//  Parameters:
//  s : vector [1,N] : input signal
//  sample rate : integer : sample rate
//  b : integer : filtration parameter
//  t : vector [1,N] : time domain
//  s_old : vector [1,N] : original signal before filtration
//  sig_new : vector [1,N] : output signal after filtration
//  f : vector [1,N] : frequency domain
//  spec_old : vector [1,N] : frequency spectrum before filtration
//  spec_new : vector [1,N] : frequency spectrum after filtration
//  
//  Description
//  Firstly, original signal is processed via FFT. The frequencies with frequency-spectrum-amplitude 
//  lower than b-times of the biggest freq-spectrum element are removed. This is useful especially 
//  when compressing audio signals.  The output contains both the old and the new signal and their 
//  frequency spectra. Moreover, the corresponding time and frequency domain vectors are returned.
//
//  Examples  
//  sample_rate=100;
//  t = 0:1/sample_rate:100;
//  N=size(t,'*');
//  s=sin(2*%pi*50*t)+sin(2*%pi*70*t+%pi/4)+grand(1,N,'nor',0,1);
//  b = 0.06;
//  [t,s_old,sig_new, f,spec_old,spec_new] = ampl_k(s,sample_rate,b);
//  figure(1);plot(t,s_old);plot(t,sig_new,'r');
//   figure(2);plot(f,spec_old);plot(f,spec_new,'r');
//
//
//  See also
//  notrend
//  alias_k
//  d_k
//  rsmp
//  pp_k
//  pz_k
//  h_k

// CODE
  t = linspace(0,length(s)/sample_rate,length(s));
  N=size(t,'*'); //number of samples
  y=fft(s);
  f=sample_rate*(-(N/2):(N/2)-1)/N; //associated frequency vector
  n=size(f,'*');
  maxik=maxi(abs(y));
  for i=1:n
  if (abs(y(i))<=(b*maxik)) then 
    yn(i)=0;
  else
    yn(i)=( y(i));
  end
  end
  spec_new = yn';
  sig_new = (ifft  (spec_new));
  spec_old = y;
  s_old = s;
  spec_new = abs(fftshift(spec_new(1:length(spec_new)/2)));
  spec_old = abs(fftshift(spec_old(1:length(spec_old)/2)));
  f = f((length(f)/2)+1:length(f));

endfunction