function [x0] = estimatex0(y,u,A,B,C,D,len) 
// Initial state estimate
//
// Calling Sequence
// [x0] = estimatex0(y,u,A,B,C,D)  
// [x0] = estimatex0(y,u,A,B,C,D,len)
// [x0] = estimatex0(y,u,ssSystem)  
// [x0] = estimatex0(y,u,ssSystem,len)
//
// Parameters
//Inputs:
//y: output data
//u: input data
//A,B,C,D: system matrices
//len: the length of the data that will be used to compute the estiamte.
//
//Outputs:
//x0: initial state estimate
//
// Description
// Estimates initial state of a discrete LTI system given IO data.
//
//The default length of the data used for the estimation is: len = min(15n/l, length(y)), where n is the system order and l is the number of outputs.
//
//The function accepts directly state-space matrices or ss structure created, for instance, by syslin() function.
//
//The function uses standard procedure with the extended observability matrix and the impulse response matrix of the system.
//
// Examples
//A = [0.8 1;0 0.6]; B = [0;1]; C = [1 0]; D = 0; x0 = [1;1]
//sys = syslin('d',A,B,C,D,x0); // discrete-time LTI
//u = rand(1,40);
//y = lsim(sys,u)+rand(40,1)/10; //noisy response
//x0est = estimatex0(y,u,sys)
//
// See also
//  subid
//  subspaceIdent
//  lsim


  [LHS RHS] = argn(0);
  if (RHS ~= 6 & RHS ~= 7 & RHS ~= 3 & RHS ~= 4)
    error('Wrong number of input arguments. Use estimatex0(y,u,A,B,C,D [,len])  or estimatex0(y,u,ssSystem [,len]).');
  end
  if (size(y,2) < size(y,1))
    y = y';  
  end
  if (size(u,2) < size(u,1))
    u = u';  
  end
  
  if (RHS <= 4)//ss structure argument
    ssSystem = A;
    try
     [A B C D] = abcd(ssSystem); 
    catch
      error('ssSystem incorectly specified.');
    end
    if (RHS == 4)
      len = B;
    end
  end
  
  if (exists('len') == 0)
    k = floor(min(15*size(A,1)/size(C,1),size(y,2)));
  else
    k = len
  end
  
  try
    S = [A B; C D];
  catch
    error('Incompatible dimensions of A,B,C,D')
  end 
  
  yn = y(:,1:k); un = u(:,1:k);
  
  m = size(B,2);
  l = size(C,1);

  O = zeros(k*l,size(A,2));
  for i = 1:k
    O((i-1)*l+1 : i*l, :) = C*A^(i-1);
  end
  
  col = zeros(k*l,m);
  col(1:l,1:m) = D;
  for i = 1:k-1
   col(i*l+1 : (i+1)*l, :) = C*A^(i-1)*B; 
  end
  H = zeros(size(col,1),k*size(col,2));
  for i = 1:k
    H((i-1)*l+1:$,(i-1)*m + 1:i*m) = col(1:$-size(D,1)*(i-1),:);
  end

  x0 = O\(yn(:) - H*un(:));
endfunction

