function [t,s_old,s_new, f,freq_old,freq_new] = h_k(s1,f_vz,fc)
//  Realizes ideal high-pass filter.
//  
//  Calling sequence  
//  [t,s_old,s_new, f,freq_old,freq_new] = h_k(s1,f_vz,fc)  
//  
//  Parameters:
//  s1: vector [1,N] : input signal
//  f_vz : integer : sampling rate
//  fc : integer : bound frequency
//  t : vector [1,N] : time domain
//  s_old : vector [1,N] : original signal before filtration
//  s_new : vector [1,N] : output signal after filtration
//  f : vector [1,N] : frequency domain
//  freq_old : vector [1,N] : frequency spectrum before filtration
//  freq_new : vector [1,N] : frequency spectrum after filtration
//  
//  Description
//  Firstly, original signal is processed via FFT. The frequency spectrum is shifted and frequencies 
//  lower than the bound frequency are zeroised. The spectrum is shifted again and processed via IFFT. 
//  The output contains both the old and the new signal and their frequency spectra. Moreover,
//  the corresponding time and frequency domain vectors are returned.
//  
//  Examples  
//  t = linspace(0,50,1000)';
//  fik1 = 8;fik2 = 3;fik3 = 1;fik4 = 5;
//  s1 = [3+10*sin(2*%pi*fik1*t) + sin(2*%pi*fik2*t) + 3*sin(2*%pi*fik3*t)+ 2*sin(2*%pi*fik4*t)]';
//  f_vz = 20; fc = 4;
//  [t,s_old,s_new, f,freq_old,freq_new] = h_k(s1,f_vz,fc);
//  figure(1);plot(t,s_old,'g');plot(t,s_new,'k');
//  figure(2);plot(f,freq_old);plot(f,freq_new,'m');
//
//
//  See also
//  notrend
//  ampl_k
//  alias_k
//  rsmp
//  pp_k
//  pz_k
//  d_k

// CODE
    
  fkvak1 = f_vz/2;
  fkvak2 = fc;
  t = linspace(0,length(s1)/f_vz,length(s1));
  f = -f_vz/2:1/((length(s1)-1)/f_vz):f_vz/2;
  freq_s1 =fftshift(fft(s1));
  f1 =(fkvak1)/(f_vz/length(s1));
  freq_new1 = freq_s1;
  freq_new1(1:length(s1)/2-f1)=0;freq_new1((length(s1)/2+f1):length(freq_s1))=0;
  f2 = fkvak2/(f_vz/length(s1)); 
  freq_new2 = freq_new1;
  freq_new2(length(s1)/2-f2:length(s1)/2+f2)=0;
  pom_kvak1 = freq_new2(1:(length(s1)/2));
  pom_kvak2 = freq_new2((length(s1)/2)+1:length(s1));
  freq_kvak = [pom_kvak2 pom_kvak1 ];
  filtered2 = ifft(freq_kvak);
  s_old = s1;
  s_new = filtered2;
  freq_old = freq_s1;  
  freq_new = freq_new2;
  f = f((length(f)/2)+1:length(f));
  freq_old = abs(freq_old((length(freq_old)/2)+1:length(freq_old)));
  freq_new = abs(freq_new((length(freq_new)/2)+1:length(freq_new)));  
endfunction