function  datnew = nanredundant(data, samplingPeriod, threshold, N1, N2)  
// Removes redundant NaN caused by non buffered sampling.
//
// Calling Sequence
// [datnew] = nanredundant(data, samplingPeriod, threshold, N1, N2)   
//
// Parameters
//   Inputs:
//     data:  measured data, first row must be the time vector
//     samplingPeriod:  expected interval between two samples
//     threshold:  usualy derived from sampling time and time of measuring of one sample from different sensors
//     N1:  starting sample
//     N2:  ending sample
//   Outputs:
//     datnew:  data without any missing data caused by non buffered sampling
// Description
//  This function was created because of non buffered sampling in an experiment. Non buffered sampling means samples from different sensors are not written at the same time (there are some time delays among these samples). The function takes the data (first vector must be the time of sampling), original sampling time, threshold (equals at least [(sampling time) - (time of record for one non buffered sample )]) and removes the redundant missing values caused by non buffered sampling.
//
// Examples
//  time = [0 27 28 29 30 58 59 60 89 90 91];
//  meas = [5 %nan %nan %nan 10 %nan 15 %nan  20 %nan 20];
//  data = [time;meas];
//  datnew = nanredundant(data,30,5)
// See also
//  nanhandle 
//  repairtime
  
// CODE  

  // parameter check
  [LHS RHS] = argn(0);
  if (RHS < 3 | RHS > 5)
    error('Wrong number of input arguments. Use nanredundant(data, samplingPeriod, threshold [,N1] [,N2]).')
  end
  
  if(~exists('data') | ~exists('samplingPeriod') | ~exists('threshold'))
  error('Input parameters undefined!'); end; 
  
  // input data dimensions
  mm = size(data,1);
  NN = size(data,2); 
  
  if(~exists('N1') | ~exists('N2')) 
    N1 = 1;
    N2 = NN; end;
    
  if(N2 <= N1)
  error('Incorect parameters. N2 must be greater than N1.');end;      
         
  if(threshold > samplingPeriod)
  error('Incorect parameters. Threshhold must be smaller than sampling time.');end;
  
  if(threshold > 3/4*samplingPeriod)
  disp('Warning: Threshhold is greater than 3/4 of the sampling time.');end;
  
  time = data(1,N1:N2)-data(1,N1);
  dat = data(:,N1:N2);
  
  // processed data dimensions 
  m = size(dat,1);
  N = size(dat,2); 
  
  s = find(diff(time)>threshold);
  ds = diff(s);
  
  timestamp = zeros(1,N);
  timestamp(s) = 1;

  
  // throw redundant nan
  for j = 2:m
    for i = 1:length(s)-1
      
      datnew(j,i) =  nanmean(dat(j,  s(i+1) :-1: (s(i+1)-ds(i)+1) ));
      
    end
    // last sample   
    datnew(j,length(s)) = nanmean(dat(j, s($):N ));
    
  end
  
  timenew = zeros(1,length(s));
  timenew = time(s);
  timenew = [timenew, mean(time((s($)+1):$))];
  timenew = floor(timenew)+data(1,N1);
  datnew = [timenew;data(2:m,1), datnew(2:m,:)] ; 

  
  
  
  endfunction