function [m, An, Bn, Cn, Dn, T, e] = physical(A, B, C, D, A0, B0, C0, D0, initial, emax, sil)
// Transfer State Space System Matrices into the desired structure by minimizing
// e = min_{theta, T} (||T.A - A0(theta).T||_F^2 + ||T.B - B0(theta)||_F^2 + ||C - C0(theta).T||_F^2)
//
// Calling Sequence
//  [An, Bn, Cn, Dn, T, e] = physical(A, B, C, D, A0, B0, C0, D0, initial, emax, sil) // 
//  [An, Bn, Cn, Dn, T, e] = physical(A, B, C, D, A0, B0, C0, D0, initial, emax) // 
//  [An, Bn, Cn, Dn, T, e] = physical(A, B, C, D, A0, B0, C0, D0) // 
// Parameters
//   Inputs:
// A,B,C,D: State Space System matrices from an identification experiment
// A,B,C,D: Required State Space System matrices structure (parameters to be estimated mark as %inf)
// initial: initial values of %inf elements in A0, B0, C0, D0
// emax: max value of error e
// sil: when sil=1, output is not generated
//   Outputs:
// m: idmodel structure (see help idmodel)
// An,Bn,Cn,Dn: Recomputed State Space System matrices in a desired form
// T: similarity transform matrix
// e: achieved value of the error
// 
// Description
// Transfer State Space System Matrices into the desired structure by minimizing
// e = min_{theta, T} (||T.A - A0(theta).T||_F^2 + ||T.B - B0(theta)||_F^2 + ||C - C0(theta).T||_F^2), where ||.||_F is the Frobenious norm. theta is the vector of unknown parameters and T is a similarity transform matrix to be found by the algorithm. The task is to find a similarity transform matrix T matching
// T.A = A0(theta).T
// T.B = B0(theta)
// C = C0(theta).T
// Note:
// Examples
//// Estimate Physical Parameters by BLACK-BOX Modeling - Ex.
//// Inverted pendulum:
//A = [-18.4268 30.1636 -15.0469 3.3101;
//       -3.3101 -5.0311 9.9923 -1.6512;
//       1.6512 -9.9923 5.0311 3.3101;
//       -3.3101 15.0496 -30.1636 18.4268];
//B = [-2.5304; 0.8296; -0.8214; 2.4391];
//C = 1e-003*[0.8118 -0.8301 -0.88 0.8282];
//D = [0];
//
//// Required Structure of matrices
//A0 = [0 1   0   0; 0 0  %inf  0; 0 0   0   1; 0 0  %inf  0];
//B0 = [0; %inf; 0; %inf];
//C0 = [1.5 0 0 0];
//D0 = [0];
//
//// Choose initial values of  %inf values in A0, B0, C0
//m = 1; M = 7; l = 2; g = 10; // known physical parameters
//ini1 = -2*m*g/(2*M+m);
//ini2 = 2*g*(M+m)/((2*M+m)*l);
//ini3 = 2/(2*M+m);
//ini4 = -1/((2*M+m)*l);
//initial = [ini1 ini2 ini3 ini4];
//
//emax = 0.0000001;
//[m,An,Bn,Cn,Dn] = physical(A, B, C, D, A0, B0, C0, D0, initial,emax)
////[An,Bn,Cn,Dn] = physical(A, B, C, D, A0, B0, C0, D0, initial)
////[An,Bn,Cn,Dn] = physical(A, B, C, D, A0, B0, C0, D0)
// 
// See also
//  subid
//  subspaceIdent
// 
// Authors
//
//Bibliography
//   Xie Liang, L. and Ljung, L.: Estimate physical parameters by Black-Box modeling
//

[LHS,RHS]=argn(); // number of arguments

// Check the arguments
if (RHS < 8) then
    error('physical needs at least 8 arguments');
end

if (RHS < 11) then
    sil = 1;
end;

if (RHS < 10) then
    emax = 0.01;
end;

AA = A0(:);
BB = B0(:);
CC = C0(:);
Syst = [AA; BB; CC];

// Equality constraints for quadprog
Aeq = []; beq = [];
for jj = 1:size(Syst,1)
    if Syst(jj) == %inf
        Aeq(jj) = 0;
        beq(jj,:) = 0;
    else
        Aeq(jj) = 1;
        beq(jj,:) = Syst(jj);
    end
end
Aeq = diag(Aeq);

// Step 1 - choose initial values for the unknown parameters in A0, B0, C0
if (RHS < 9) then // initial values of unknown parameters are not specified (set equal to 1)
    for ii = 1:size(Syst,1)
        if Syst(ii) == %inf
            Syst(ii) = 1;
        end
    end
else // initial values of unknown parameters are specified
    initpos = 1;
    for ii = 1:size(Syst,1)
        if Syst(ii) == %inf
            Syst(ii) = initial(initpos);
            initpos = initpos + 1;
        end
    end 
end;

sizT = size(A,1);
A0 = matrix(Syst([1:sizT^2]),sizT,sizT);
B0 = matrix(Syst([sizT^2+1:sizT^2+(size(B,1)*size(B,2))]),size(B,1),size(B,2));
C0 = matrix(Syst([sizT^2+(size(B,1)*size(B,2))+1:$]),size(C,1),size(C,2));


//// Step 2 - minimize the criterion (6) by solving for T
e = %inf; kk = 1; terminate = 1;
I = eye(sizT,sizT);
while e($) > emax & terminate == 1
    LhsT = [kron(A',I) - kron(I,A0); kron(B',I); -kron(I,C0)];
    RhsT = [zeros(size(B,1)*size(C,2),1); B0(:); -C(:)];
    Tvec = LhsT\RhsT;
    T = matrix(Tvec,sizT,sizT);

    // Step 3 - minimize the criterion (6) by solving for theta  
    Lhs = [kron(T',I) zeros(sizT^2,size(B,1)) zeros(sizT^2,size(C,2));
           zeros(size(B,1),sizT^2) eye(size(B,1),size(B,1)) zeros(sizT,size(C,2));
           zeros(size(C,2),size(T,1)^2) zeros(size(B,1),size(B,1)) kron(T',eye(size(C,1),size(C,1)))];
    Rhs = [kron(I,T)*A(:); kron(eye(size(B,2),size(B,2)),T)*B(:); C(:)];
    // Quadprog
    H=2*Lhs'*Lhs;
    f=-2*Lhs'*Rhs;
    //thvec = quadprog(H,f,[],[],Aeq,beq); // matlab
    thvec = qpsolve(H,f,Aeq,beq,[],[],size(Aeq,2));
    // New matrices
    A0 = matrix(thvec([1:sizT^2]),sizT,sizT);
    B0 = matrix(thvec([sizT^2+1:sizT^2+size(B,1)*size(B,2)]),size(B,1),size(B,2));
    C0 = matrix(thvec([sizT^2+1+size(B,1)*size(B,2):size(thvec,1)]),size(C,1),size(C,2));

    // Step 4 - check the value of the error
    e(kk) = norm(([kron(A',I) - kron(I,A0); kron(B',I); -kron(I,C0)]*T(:)) -...
        ([zeros(size(B,1)*size(C,2),1); B0(:); -C(:)]))^2;
    kk = kk + 1;
    if kk > 1000 // protection against infinity loop
        terminate = -1;   
    end
end

if sil == 0
    plot(e,'Linewidth',2);
    hold on;
    plot([0 kk],[emax emax],'r--','Linewidth',2)
    title('Error of estimate')
    ylabel('Error of estimate');
    legend('e','emax')
    disp('   Value of the error e =   ');
    disp(e($))
    disp('   Max error emax =   ');
    disp(emax)
end

if terminate == -1
    disp('physical: emax cannot be reached. Please change emax value or select another initial estimate of unknown parameters.');
else
    disp('physical: emax was reached.');
end

An = A0;
Bn = B0;
Cn = C0;
Dn = D0;
e = e($);

m = idmodel();
m.A = An;
m.B = Bn;
m.C = Cn;
m.D = Dn;
m.algorithm = 'physical';

endfunction


