function [m, a, b, vare, P, nu] = rarx(y, u, na, nb, nk, phi, theta0, P0, nu0, vare0)
// Recursive ARX model estimation.
//
// Calling Sequence
//   [a, b, vare] = rarx(y,u,na,nb) //nk = 0, phi = 1
//   [a, b, vare] = rarx(y,u,na,nb,nk) //phi = 1
//   [a, b, vare] = rarx(y, u, na, nb, nk, phi)
//   [a, b, vare, P, nu] = rarx(y, u, na, nb, nk, phi, theta0, P0)
//   [a, b, vare, P, nu] = rarx(y, u, na, nb, nk, phi, theta0, P0, nu0, vare0)
//
// Parameters
//   Inputs:
//           y: vector of outputs
//           u: vector of inputs
//           na: number of poles
//           nb: number of zeros + 1
//           nk: input-output time delay
//           theta0: initial estimate of the polynomials a(d), b(d): theta0 = [a0 b0]
//           P0: The covarience matrix of the initial estimate of [-a0($:-1:2), b0($:-1:1)]
//           nu0: initial index in the recursion - acts as a weight of the vare0 estimate
//           vare0: Initial estimate of the noise varience
//           phi: exponential forgetting factor: 1 -> no forgetting, common choice: phi in [0.9, 1]
//   Outputs:
//            m: Uniform Identification Toolbox model structure (see help idmodel)
//            a: coeficients of a(d)
//            b: coeficients of b(d)
//            vare: the varience of e
//            P: normed covarience matrix of the estimate, i.e. P_true / vare.
//            nu: index in the recursion
//
// Description
//   Computes recursively an arx model of a given order, and the input noise varience. An initial estimate theta0, P0 and the forgetting factor phi are optinal parameters. The optional parameters nu0, vare0, eps0 can be used for recursive identification.
//
//Standard arx model structure is used:
//
//      a(d)y(t) = b(d)u(t) + e(t), where
//
//      a(d) = 1 + a_1*d + ... + a_na*d^na  
//
//      b(d) = b_0*d^nk + ... + b_nb * d^(nk+nb)
//
//      with d being the unit delay operator: dx(t) = x(t-1).
//
//The length of the input data vectors u, y must be at least max(na+1, nb+nk).
//
//
// Examples
//data = read('../demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[m] = rarx(y,u,3,3,2);
//[fit, yp] = valPred(m.a, m.b, [], yv, uv);
//plot(yv);
//plot(yp,'-r');
//
// See also
//  armax
//  arx
//  arIdent
//  valPred
//  valSim
//  subid
//  subspaceIdent

  [LHS RHS] = argn(0);
  if (RHS < 4)
    error('Not enough input arguments, at least four arguments are needed: y, u, na, nb');
  end
  if (RHS == 4)
    nk = 0;
  end
  if (RHS == 7)
    error('The covariance matrix of theta0 must be specified.')
  end
  if (RHS <= 6)
    th = zeros(na+nb,1);
    P = 100*eye(size(th,1), size(th,1));
  end
  if (RHS >= 8)
    th = [mtlb_fliplr(-theta0(2:na+1))'; mtlb_fliplr(theta0(na+nk+2:$))']; 
    P = P0;
  end 
  if (RHS == 9)
    error('An initial estimate of the noise varience must be provideed with nu0.')
  end
  if (RHS == 10)
    nu = nu0;
    nus2 = nu*vare0;
  end
  if (RHS <= 8)
    nu = 0;
    nus2 = nu*4; //vare0 = 4  
  end
  if(RHS < 6)
    phi = 1;
  end
  
  if (size(y,1) ~= size(u,1) | size(y,2) ~= size(u,2))
    error('y and u must be of the same size.')
  end
  if (size(y,1) < size(y,2))
    y = y';
    u = u';
  end
  
  //---------------Compute the estimate------------------------
  
  for t = max(na+1,nb+nk):size(y,1)
    z = [y(t-na:t-1,1)' u(t-nb-nk+1:t-nk,1)']';    
    nus2 = nus2 + (y(t) - z'*th)^2/(1 + z'*P*z);
    P = (1/phi) * (P - P*z*z'*P/(1 + z'*P*z)); //covarience update
    th = th + P*z*(y(t)-z'*th); //parameters update
    nu = nu + 1;
  end
    
  a = [1 -mtlb_fliplr(th(1:na)')];
  b = [zeros(1,nk) mtlb_fliplr(th(na+1:$)')];
  vare = nus2 / nu;
  
m = idmodel();
m.a = a;
m.b = b;
m.vare = vare;
m.P = P;
m.algorithm = 'RLS';
m.na = na;
m.nb = nb;
m.nk = nk;

endfunction