function [m,mstruc] = selstruc(mo,uv,yv,yi)
// Select model order for single-output ARX/ARMAX models.
//
// Calling Sequence
// [m] = selstruc(mo,uv,yv,yi)
// [m,mstruc] = selstruc(mo,uv,yv,yi)
// Parameters
//   Inputs:
// mo: ARX/ARMAX (in idmodel structure form - see help idmodel) model to be reduced
// uv: input validation data
// yv: output validation data
// yi: output identification data
//   Outputs:
// m: reduced ARX/ARMAX idmodel structure (in idmodel structure form - see help idmodel)
// mstruc: structure with the modified parameters of theta
//       mstruc.lambda:   parameter of model complexity
//       mstruc.w:        table of weighting vectors of the original parameters
//       mstruc.thetaNew: table of the new parameters
//       mstruc.A:        table of the new coeficients of the polynomial A
//       mstruc.B:        table of the new coeficients of the polynomial B
//       mstruc.C:        table of the new coeficients of the polynomial C
// 
// Description
// Select model order for single-output ARX models by the use of Nonnegative Garrote (NNG) method.
//
// Examples
//N = 2000;
//bound = N/2;
//na = 10;
//nb = 5;
//nk = 0;
//// Fan and Plate process:
////a0 = [1 -1.25 0.4375 -0.3594 0.1719 0.3125 -0.2764 0.1360 -0.0769 0.0137]; 
////b0 = [1 0.25 -0.25];
//a0 = [1 0.1 -0.2 0.04 -0.03 ]; 
//b0 = [1 2 -0.04 -0.009];
//
//u = rand(N,1,'normal'); // input
//[yr] = lsim(a0,b0,u);
//y = yr + sqrt(0.1)*rand(N,1,'normal');
//yi = y(1:bound);
//ui = u(1:bound);
//yv = y(bound+1:$); uv = u(bound+1:$); // validation data
//
//
//example = 1; // 1 = ARX, 2 = ARMAX
//
//select example
//case 1
////------------------ARX model estimate------------------
//[marx,a1,b1,FI] = arx(yi,ui,na,nb,nk);
//[marx.fit, ys] = valSim(a1, b1, yv, uv);
//
//// Initial model
//figure
//plot(yv);
//plot(ys,'r');
//title(string(marx.fit),'fontsize',3);
////------------------ARX model reducing------------------
////mo = idmodel();
////mo.a = marx.a; mo.b = marx.b;
//[m1,mstruc] = selstruc(marx,uv,yv,yi);
//
//// Reduced model
//[m1.fit, ys] = valSim(m1.a, m1.b, yv, uv);
//figure
//plot(yv);
//plot(ys,'r');
//title(string(m1.fit),'fontsize',3);
//
//case 2
////------------------ARMAX model estimate------------------
//nc = 10;
//[marmax,a1,b1,FI] = armax(yi,ui,na,nb,nc,nk);
//[marmax.fit, ys] = valPred(marmax.a, marmax.b, marmax.c, yv, uv);
//
//// Initial model
//figure
//plot(yv);
//plot(ys,'r');
//title(string(marmax.fit),'fontsize',3);
////------------------ARMAX model reducing------------------
////mo2 = idmodel();
////mo2.a = marmax.a; mo2.b = marmax.b; mo2.c = marmax.c;
//[m2,mstruc] = selstruc(marmax,uv,yv,yi);
//
//// Reduced model
//[m2.fit, ys] = valPred(m2.a, m2.b, m2.c, yv, uv);
//
//figure
//plot(yv);
//plot(ys,'r');
//title(string(m2.fit),'fontsize',3);
//end // end select
//
// See also
//  arx
//  armax
//  arIdent
// 
// Authors
// 
// 
//Bibliography
//   Lyzell, Ch.: Initialization Methods for System Identification
//   Lyzell, Ch.: The Use of Non-Negative Garrote for Order Selection of arx Models
//


[LHS RHS] = argn(0);
if (RHS < 4)
    error('Not enough input arguments. Use [m] = selstruc(mo,uv,yv,yi)');
  end
  
if (size(yv,1) ~= size(uv,1) | size(yv,2) ~= size(uv,2))
    error('yv and uv must be of the same size.')
end

if (size(yv,1) < size(yv,2))
    y = y';
    u = u';
end

if mo.c == [] // ..... ARX .....
    a = mo.a; b = mo.b;
    na = size(a,2)-1; nb = size(b,2);

    theta = [a b]';
    //// Reducing the order of the ARX model by NNG algorithm:
    // Inequality constraints
    if size(na,1) == 1 // SISO (MISO)
        Aconstr1l = [eye(na,na); zeros(1,na)] + [zeros(1,na); -eye(na,na)];
        Aconstr1 = [kron(eye(size(na,1)),Aconstr1l) zeros(size(Aconstr1l,1),nb)];
        Aconstr2 = [eye(nb,nb); zeros(1,nb)] + [zeros(1,nb); -eye(nb,nb)];
        Aconstr2 = [zeros(nb+1,sum(na)) Aconstr2];
        Aconstr = [Aconstr1; Aconstr2];
        bconstr = kron(ones(size(na,1),1),[1; zeros(na,1)]);
        bconstr = [bconstr; [1; zeros(nb,1)]];
        
    end

    // calling the NNG algorithm for reducing the order of model
    Y = yi(max(na+1,nb+nk):$,:);
    THETA = diag(theta(2:$));
    Q = 2*THETA*(FI)'*FI*THETA;
    f = -2*THETA*FI'*Y;
    N = size(yi,1);
    
    FI = zeros(N-max(na+1,nb+nk)+1, na+nb);
    for q = 1:na
      i = na-q+1;
      FI(:, (i-1)+1 : i) = yi(max(na,nb+nk-1)-i+1 : N-i,:);
    end
    for q = 1:nb
      i = nb-q+1;
      FI(:, (i-1)+1+na : i+na) = ui(max(na,nb+nk-1)-i-nk+2 : N-i-nk+1,:);
    end

    mstruc = nng(FI, theta, Aconstr, bconstr, na, nb, [], nk, yi); 


    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%SIMULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    // TODO minor changes for MISO

    for ii = 1:size(mstruc.thetaNew,2) // simulation of all models & computing fit
        cond = isnan(sum(mstruc.thetaNew(:,ii)));
        if cond ~= 1
            [mstruc.fit(:,ii),ys] = valSim(mstruc.A(:,ii)', mstruc.B(:,ii)',yv,uv);
    // mstruc.fit(:,ii) = 100*(1-norm(yv-ys,'fro')/norm(yv-mean(yv),'fro'));
        else
            mstruc.fit(:,ii) = %nan;
        end
    end

    for qq=1:length(mstruc.fit) // the range for plotting
        if mstruc.fit(qq)>10
            lambdaLength = qq;
        else
            break;
        end
    end
    
end // ..... ARX END.....

if mo.c ~= [] // ..... ARMAX.....
    a = mo.a; b = mo.b; c = mo.c;
    theta = [a b c]';
    na = size(a,2)-1; nb = size(b,2); nc = size(c,2)-1;
    Aconstr1 = [eye(na,na); zeros(1,na)] + [zeros(1,na); -eye(na,na)];
    Aconstr1 = [Aconstr1 zeros(na+1,nb+nc)];
    Aconstr2 = [eye(nb,nb); zeros(1,nb)] + [zeros(1,nb); -eye(nb,nb)];
    Aconstr2 = [zeros(nb+1,na) Aconstr2 zeros(nb+1,nc)];
    Aconstr3 = [eye(nc,nc); zeros(1,nc)] + [zeros(1,nc); -eye(nc,nc)];
    Aconstr3 = [zeros(nc+1,na+nb) Aconstr3];
    Aconstr = [Aconstr1; Aconstr2; Aconstr3];
    bconstr = [1 zeros(1,na) 1 zeros(1,nb) 1 zeros(1,nc)]';

    [fit, ys] = valPred(mo.a, mo.b, mo.c, yv, uv);
    e = yi - ys;
    N = size(yi,1);
    FI = zeros(N-max(na+1,nb+nk)+1, na+nb);
    for q = 1:na
      i = na-q+1;
      FI(:, (i-1)+1 : i) = yi(max([na,nb+nk-1,nc])-i+1 : N-i,:);
    end
    for q = 1:nb
      i = nb-q+1;
      FI(:, (i-1)+1+na : i+na) = ui(max([na,nb+nk-1,nc])-i-nk+2 : N-i-nk+1,:);
    end
    for q = 1:nc
      i = nc-q+1;
      FI(:, (i-1)+1+na+nb : i+na+nb) = e(max([na,nb+nk-1,nc])-i-nk+2 : N-i-nk+1,:);
    end 
    
    theta_ab = theta([2:na+1+nb]);
    theta_c = theta([na+1+nb+2:$]);
    theta_abc = [theta_ab; theta_c];
    THETA = diag(theta_abc);
    Y = yi(max(na+1,nb+nk):$,:);
    Q = 2*THETA*(FI)'*FI*THETA;
    f = -2*THETA*FI'*Y;
    
    

    // calling the NNG algorithm for reducing the order of model
    mstruc = nng(FI, theta, Aconstr, bconstr, na, nb, nc, nk, yi); 
    
        for ii = 1:size(mstruc.thetaNew,2) // simulation of all models & computing fit
        cond = isnan(sum(mstruc.thetaNew(:,ii)));
        if cond ~= 1
//      [mstruc.fit(:,ii),ys] = valSim(mstruc.A(:,ii)', mstruc.B(:,ii)',yv,uv);
   [mstruc.fit(:,ii),ys] = valPred(mstruc.A(:,ii)', mstruc.B(:,ii)', mstruc.C(:,ii)', yv, uv);
    //        mstruc.fit(:,ii) = 100*(1-norm(yv-ys,'fro')/norm(yv-mean(yv),'fro'));
        else
            mstruc.fit(:,ii) = %nan;
        end
    end

    for qq=1:length(mstruc.fit) // the range for plotting
        if mstruc.fit(qq)>10
            lambdaLength = qq;
        else
            break;
        end
    end
    
end // ..... ARMAX END.....

////////////////////////////////////////////////////////////////////////////////

figure // fullfilment of coefficients
subplot(211)
spy(mstruc.thetaNew(:,[1:lambdaLength]));
xlabel('Index of break points \lambda');
ylabel('thetaNew');
title('Please select a new reduced model according to the Fit Factor in 2nd graph by CLICK ','fontsize',2);

// Fit
subplot(212)
plot(1:lambdaLength, mstruc.fit(1:lambdaLength));
xlabel('Index of break points \lambda');
ylabel('Model FIT in %');

disp('      Please, select a new model by left mouse CLICK');
[xx]=locate(1); // capture mouse click
close

////////////////////////////////////////////////////////////////////////////////

// New selected model
if mo.c == []
    Anew = mstruc.A(:,round(xx(1)))';
    Bnew = mstruc.B(:,round(xx(1)))';
    [fitNew, ys] = valSim(Anew, Bnew, yv, uv);
    anew = Anew;
    bnew = Bnew;
    m = idmodel();
    m.a = anew;
    m.b = bnew;
else
    Anew = mstruc.A(:,round(xx(1)))';
    todel=find(Anew);
    Anew(todel($)+1:$) = [];
    clear("todel");
    Bnew = mstruc.B(:,round(xx(1)))';
    todel=find(Bnew);
    Bnew(todel($)+1:$) = [];
    clear("todel");
    Cnew = mstruc.C(:,round(xx(1)))';
    todel=find(Cnew);
    Cnew(todel($)+1:$) = [];
    clear("todel");
    [fitNew, ys] = valPred(Anew, Bnew, Cnew, yv, uv);
    anew = Anew;
    bnew = Bnew;
    cnew = Cnew;
    m = idmodel();
    m.a = anew;
    m.b = bnew;
    m.c = cnew;    
end

m.fit = fitNew;
m.algorithm = 'Nonnegative Garrote';

endfunction
