function [x1,p1,y1] = srcf(y,u,x0,p0,A,B,C,D,qn,rn)
// Kalman filter SRCF implementation (Square-root covariance filter) by
// Verhaegen, M. and Verdult, V. Filtering and system identification:
// A Least Squares Approach. Cambridge University Press, 2007.
//
// Calling Sequence
// [x1,p1,y1] = srcf(y,u,x0,p0,A,B,C,D,qn,rn)
// Parameters
//   Inputs:
//     y:        observation of output signal
//     u:        observation of input singal
//     x0:       state estimate at k=0 based on data up to k=-1
//     p0:       error covariance at k=0 based on data up to k=-1
//     A,B,C,D:  system matrices
//     qn,rn:    covariances of dynamic and observation noise
//   Outputs:
//     x1:       state estimate at k=1 based on data up to k=0
//     p1:       error covariance at k=1 based on data up to k=0
//     y1:       output of a system at k=1 based on data up to k=0 computed from the state estimate x1: y1 = Cx1 + Du
// 
// Description
// Considering the stochastic system:
//	x(k+1) = Ax(k) + Bu(k) + w(k)
//	  y(k) = Cx(k) + Du(k) + v(k).
//
// The process and measurement noise is represented as
// [v(k)] = [R(k)^(1/2)         0     ] * [v~(k)]		[v~(k)] = (0, I_{l+n})
// [w(k)]	[X(k)          Qx(k)^(1/2)]	  [w~(k)] with  [w~(k)]
// 			|-------------------------|
//						||
//					   L(k)
// where X(k) = S(k)R(k)'^(-1/2)
//		 Qx(k) = Q(k)-S(k)R(k)^{-1}S(k)'.
//
// Given the system matrices A, B, C, D and the covariance matrices Q, R
// for time instants k = 0,1,2,.. and the initial state x(0|-1) and the square
// root of its covariance matrix P(0|-1)^(1/2), the algorithm apply the
// orthogonal transformation Tr such that:
// [C(k)P(k|k-1)^{1/2}		R(k)^{1/2}      0	 	]	*	Tr = 
// [A(k)P(k|k-1)^{1/2} 		   X(k)     Qx(k)^{1/2}	]        
// 		= [Re(k)^{1/2}  		0 			0] 
//   	  [G(k) 		P(k+1|k)^{1/2} 		0] = ortho.
// Update the one-step ahead prediction as
// x(k+1|k) = A(k)x(k|k-1) + Bu(k) + GRe(k)^(-1/2)(y(k) - C(k)x(k|k-1)).
//
// Examples
//y = 2;
//u = 1;
//x0 = [1;1];
//p0 = eye(2,2)
//A = [1 0; 0 1];
//B = [1;-2];
//C = [1 0];
//D = 0;
//qn = 1.5;
//rn = 0.5;
//[x1,p1,y1] = srcf(y,u,x0,p0,A,B,C,D,qn,rn)
//
// See also
//  arx
//  armax
//  arIdent
//  subid
//  subspaceIdent
// 
// Authors
// 
// Bibliography
//    Verhaegen, M. and Verdult, V. Filtering and system identification:
//    A Least Squares Approach. Cambridge University Press, 2007.

x = x0;

h = [C;zeros(size(C,2)-1,size(C,2))];
r = rn*eye(size(h,2),size(h,2));
q = qn*eye(size(A,1),size(A,1));
P = p0;

//---------------- SRCF ----------------
a = max(size(x));
b = max(size(y(1)));
c = [chol(r)',0*r];
d = [0*q,chol(q)'];
dummy = [h*P,c;A*P,d];
[QQ,RR] = qr(dummy')';
sqrtRe = RR(1:b,1:b); // Re(k)^(1/2)
G = RR(a+1:$,1:b); // G(k)
P = RR(a+1:a+a,a+1:a+a); // P(k+1|k)^(1/2)
x1 = A*x + B*u + G*sqrtRe^(-1)*(y-C*x); // state update
p1 = P*P'; // errcov = P(k+1|k)^(1/2)*(P(k+1|k)^(1/2))'
y1 = C*x + D*u; // output

endfunction
