function [m,A,B,C,D,K,Ro,AUX,ss] = subid(y,u,i,dmat,n,AUXin,W,sil)
//Combined subspace identifiacation.
//Calling Sequence
//   [m] = subid(y,u,i,dmat)
//   [m,A,B,C,D,K,Ro] = subid(y,u,i,dmat)
//   [m,A,B,C,D,K,Ro,AUX,ss] = subid(y,u,i,dmat,n,AUXin,W,sil)
// 
//Parameters
//   Inputs:
//           y: matrix of measured outputs
//           u: matrix of measured inputs (for stochastic systems: u = [])
//           i: number of block rows in Hankel matrices
//           dmat: dmat = 1 for nonzero matrix D, dmat = 0 for zero matrix D
//           n:    optional order estimate (default [])
//                 if not given, the user is prompted for the order
//           AUX:  optional auxilary variable to increase speed (default [])
//           W:    optional weighting flag
//                      SV:    Singular values based algorithm 
//                             (default for systems with input u)
//                      CVA:   Canonical variate based algorithm
//                             (default for systems without input u)
//          sil: when sil=1, output is not generated
//   Outputs:
//           m: idmodel structure (see help idmodel)
//           A,B,C,D,K,Ro: Combined state space system matrices
//           ss:   column vector with singular values
//           AUX:  optional auxilary variable
//Description
//
//           
//                  x_(k+1) = A x_k + B u_k + K e_k
//     
//                    y_k   = C x_k + D u_k + e_k
//
//                 cov(e_k) = Ro
//                 
//           For deterministic systems: K = Ro = []
//
//           For stochastic systems:    B = D = []
//
//              (i * no. of outputs) is the max. order that can be estimated. Typically: i = 2 * (max order)/(no. of   outputs)
//
//Examples
//data = read('../demos/dryer.dat',-1,2); //data = [u y]
//y = data(1:400,2); u = data(1:400,1); //identification data
//yv = data(401:$,2); uv = data(401:$,1); //validation data
//[m] = subid(y,u,10,1,3,[],[],1); //i = 10, n = 3, nonzero D
//[m] = subid(y,u,10,0,3,[],[],1); //i = 10, n = 3, zero D
//m.x0 = estimatex0(yv,uv,m.A,m.B,m.C,m.D);
//[ys x] = lsim(m.A,m.B,m.C,m.D,uv,m.x0); // simulation
//plot(yv);
//plot(ys,'-r')
//
// See also
//  subspaceIdent
//  arx
//  rarx
//  arIdent
//  valPred
//  valSim
// 
//Bibliography
//           Overschee, DeMoor: Subspace identification for linear systems
//           Stochastic algorithm:   Figure 3.13 page 90 (positive)
//           Combined algorithm:     Figure 4.8 page 131 (robust)
//


[lhs,rhs]=argn();//number of arguments
if (rhs < 8) then
    sil = 0;
end;

// Check the arguments
if (rhs < 4) then
    error('subid needs at least three arguments');
end
if (rhs < 5) then
    n = [];
end
if (rhs < 6) then
    AUXin = [];
end


// Deterministic or stochastic ID?
if isempty(u) then
    ds_flag = 2; 		// Stochastic
else
    ds_flag = 1; 		// Deterministic
end;  

// Give W its default value
if (rhs < 6) then
    W = [];
end;

if isempty(W) then
  if (ds_flag == 1) then
      W = 'SV'; 		// Deterministic identification: default to SV 
  else
      W = 'CVA';  // Stochastic identification: default to CVA
  end;
end;


// Turn the data into row vectors and check
[l,ny] = size(y);
if (ny < l) then
    y = y';
    [l,ny] = size(y);
end
if (i < 0) then
  error('Number of block rows should be positive');
end;
if (l < 0) then 
  error('Need a non-empty output vector');
end;

if (ds_flag == 1) then //deterministic
  [m,nu] = size(u);
  if (nu < m)
      u = u';
      [m,nu] = size(u);
  end
  if (m < 0) then
    error('Need a non-empty input vector');
  end;
  if (nu ~= ny) then
      error('Number of data points different in input and output');
  end;
else
  m = 0;
end;
if ((ny-2*i+1) < (2*l*i)) then
    error('Not enough data points');
end;


// Check the weight to be used
Wn = 0;
if length(W) == 2 then
   if (prod(W == 'SV') | prod(W == 'sv') | prod(W == 'Sv')) then
    Wn = 1; 
    if ds_flag == 1 then //deterministic
        Waux = 2;
    else
        Waux = 3; //stochastic
    end

  end;
end;

if (length(W) == 3) then
   if (prod(W == 'CVA') | prod(W == 'cva') | prod(W == 'Cva')) then
    Wn = 2;
    if ds_flag == 1 then//deterministic
        Waux = 3;
    else
        Waux = 1;//stochastic
    end

  end;
end;

if Wn == 0 then
    error('W should be SV or CVA');
end;
W = Wn;

j = ny-2*i+1;//number of columns in the Hankel matrices

// Check compatibility of AUXin
// AUX is used when an iterative identification for iterative assessment of system order is performed
if ds_flag == 1 then
    Uaux = u(1,1);
else
    Uaux = [];
end

[AUXin,Wflag] = checkAUX(AUXin,i,Uaux,y(1,1),ds_flag,Waux); 



// R factor
if isempty(AUXin) then
  Y = blockhank(y/sqrt(j),2*i,j);  // Output block Hankel
  //mydisp(sil,"      Computing ... R factor");
  if ds_flag==1 then //deterministic
    U = blockhank(u/sqrt(j),2*i,j);  // Input block Hankel
    R = triu(qrhouse([U;Y]'))';
    clear("U","Y") 
  else //stochastic
    R = triu(qrhouse(Y'))';
    clear("Y")
  end;
  R = R(1:2*i*(m+l),1:2*i*(m+l)); 	// Truncate
else
  R = AUXin(2:2*i*(m+l)+1,1:2*(m+l)*i);
  bb = 2*i*(m+l)+1;
end



//%%%%%%%%%%%%ALGORITHM%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// **************************************
//               STEP 1 
// **************************************

mi2  = 2*m*i;
if isempty(AUXin) | Wflag==1 then
  Rf = R((2*m+l)*i+1:2*(m+l)*i,:); 	// Future outputs (R factor of Yf)
  Rp = [R(1:m*i,:);R(2*m*i+1:(2*m+l)*i,:)]; // Past (inputs and) outputs (R factor of Wp)
  if (ds_flag == 1) then//deterministic
    Ru  = R(m*i+1:2*m*i,1:mi2); 	// Future inputs (R factor of Uf)
    Rfp = [Rf(:,1:mi2) - (Rf(:,1:mi2)/Ru)*Ru,Rf(:,mi2+1:2*(m+l)*i)]; // Perpendicular Future outputs (R factor of Yfp)
    Rpp = [Rp(:,1:mi2) - (Rp(:,1:mi2)/Ru)*Ru,Rp(:,mi2+1:2*(m+l)*i)]; // Perpendicular Past (R factor of Wpp)
  end;
end


// The oblique projection: (6.1, page 166)
// obl/Ufp = Yf/Ufp * pinv(Wp/Ufp) * (Wp/Ufp): The extra projection on Ufp (Uf perpendicular) tends to give better
// numerical conditioning (see algo page 131)

if isempty(AUXin) then
  if ds_flag == 1 then//deterministic
    // Rank check (SVD takes too long)
    //This check is needed to avoid rank deficiency warnings
    if (norm(Rpp(:,(2*m+l)*i-2*l:(2*m+l)*i),'fro')) < 1e-10 then
      Ob  = (Rfp*pinv(Rpp')')*Rp; 	// Oblique projection
    else
      Ob = (Rfp/Rpp)*Rp;
    end;
  else    
    Ob = [Rf(:,1:l*i),zeros(l*i,l*i)]; // is the same projection as: Ob  = (Rf/Rp)*Rp; 
  end;
else
  Ob = AUXin(bb+1:bb+l*i,1:2*(l+m)*i);//Ob from AUXin
  bb = bb+l*i;
end





// **************************************
//               STEP 2 
//**************************************

// Compute the SVD
if isempty(AUXin) | Wflag == 1 then
  mydisp(sil,'      Computing ... SVD');
  // Compute the matrix WOW we want to take an SVD of W = 1 (SV), W = 2 (CVA)
  if ds_flag == 1 then//deterministic 
    // Extra projection of Ob on Uf perpendicular
    WOW = [Ob(:,1:mi2) - (Ob(:,1:mi2)/Ru)*Ru,Ob(:,mi2+1:2*(m+l)*i)];
  else // stochastic
    WOW = Ob;
  end;
  
  if W == 2 then
    W1i = triu(qrhouse(Rf'));
    W1i = W1i(1:l*i,1:l*i)';
    WOW = W1i\WOW;
  end;
  
  [U,S,V] = svd(WOW);
  
  if W == 2 then //CVA
      U = W1i*U;
  end; 		
  
  ss = diag(S);
  clear V S WOW
else
  U = AUXin(bb+1:bb+l*i,1:l*i);
  ss = AUXin(bb+1:bb+l*i,l*i+1);
end




// **************************************
//               STEP 3 
// **************************************

// Determine the order from the singular values

if isempty(n) then

  if W==2 then//CVA
    bar(1:l*i,real(acos(ss))*180/%pi);
    hand=gca();
    hand.auto_ticks=["on" "on" "on"];
    title("Principal Angles");
    ylabel("degrees");
    delete(gca())
  else
   //bar(1:l*i,ss);
    h=gca();
    xx=1:l*i;
    yy=ss+10^(floor(log10(min(ss))));
    h.data_bounds=[0.0001,length(ss)+1,10^(floor(log10(min(ss)))),10^(ceil(log10(max(ss))))];//visible data
    h.log_flags="nl";//logarithic y-axis
    h.auto_ticks=["on" "on" "on"];
    h.grid=[10,10];
    plot(xx,yy);  
    g=get("hdl");//access to current entity
    gg=g.children;//access to polyline
    gg.thickness=3;//line thickness
    title("Singular Values");
    xlabel("Order");
  end;

  
  n = 0;
  while (n<1) | (n > l*i-1)
    n = input("      System order ? ");
     if isempty(n) then
        n = -1;
     end;
  end;
  mydisp(sil," ")
end;

U1 = U(:,1:n);// Determine U1




// **************************************
//               STEP 4 
// **************************************

// Determine gam and gamm
gam  = U1*diag(sqrt(ss(1:n)));
gamm = gam(1:l*(i-1),:);

gammac = [gam(l+1:$,:);zeros(l,size(gam,2))] ;
// The pseudo inverses
gam_inv = pinv(gam);// Pseudo inverse
gamm_inv = pinv(gamm);// Pseudo inverse


// **************************************
//               STEP 5 
// **************************************

// Determine the matrices A and C
mydisp(sil,"      Computing ... System matrices A,C (Order "+string(n)+")");

Rhs = [  gam_inv*R((2*m+l)*i+1:2*(m+l)*i,1:(2*m+l)*i),zeros(n,l) ; R(m*i+1:2*m*i,1:(2*m+l)*i+l)];
Lhs = [        gamm_inv*R((2*m+l)*i+l+1:2*(m+l)*i,1:(2*m+l)*i+l) ; R((2*m+l)*i+1:(2*m+l)*i+l,1:(2*m+l)*i+l)];


// Solve least square
sol = Lhs/Rhs;

// Extract the system matrices A and C
A = sol(1:n,1:n);
C = sol(n+1:n+l,1:n);

res = Lhs - sol*Rhs; 
			
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//%%   Recompute gamma from A and C
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gam = C;
for k = 2:i
  gam((k-1)*l+1:k*l,:) = gam((k-2)*l+1:(k-1)*l,:)*A;
end;
gamm = gam(1:l*(i-1),:);      
gam_inv = pinv(gam);
gamm_inv = pinv(gamm);

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//%%   Recompute the residua with the new gamma
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Rhs = [  gam_inv*R((2*m+l)*i+1:2*(m+l)*i,1:(2*m+l)*i),zeros(n,l) ; R(m*i+1:2*m*i,1:(2*m+l)*i+l)];
Lhs = [ gamm_inv*R((2*m+l)*i+l+1:2*(m+l)*i,1:(2*m+l)*i+l) ; R((2*m+l)*i+1:(2*m+l)*i+l,1:(2*m+l)*i+l)];

// **************************************
//               STEP 6 
// **************************************

if ds_flag==2 then //deterministic
  B = [];
  D = [];
else
  mydisp(sil,"      Computing ... System matrices B,D (Order "+string(n)+")");
  // P and Q as on page 125
  P = Lhs - [A;C]*Rhs(1:n,:);
  P = P(:,1:2*m*i);
  Q = R(m*i+1:2*m*i,1:2*m*i); 		//Future inputs

  // L1, L2, M as on page 119
  L1 = A * gam_inv;
  L2 = C * gam_inv;
  M  = [zeros(n,l),gamm_inv];//zeros are due to first iteration in computation of N...see N=[M(:,k-1)...]
  X  = [eye(l,l),zeros(l,n);zeros(l*(i-1),l),gamm];

  totm = 0;
   for k=1:i //HERE IS THE COMPUTATIONAL BOTTLENECK OF THE ALGORITHM!
    // Calculate N and the Kronecker products (page 126)
    N = [[M(:,(k-1)*l+1:l*i)-L1(:,(k-1)*l+1:l*i),zeros(n,(k-1)*l)];[-L2(:,(k-1)*l+1:l*i),zeros(l,(k-1)*l)]];
  
    if k==1 then
      N(n+1:n+l,1:l) = eye(l,l) + N(n+1:n+l,1:l);
    end;
    N = N*X;
      totm = totm + kron(Q((k-1)*m+1:k*m,:)',N);
  end;

  // Solve Least Squares
  P = P(:);
  
  if dmat==0
      // NEW zero matrix D - change of  P(:)  and  totm  matrices
      newtotm = [];
      for k = 1:m
        newtotm = [newtotm,totm( : , k*(l+n)-n+1:k*(l+n) )]; // selected columns
      end
      totm = newtotm;
      // end of zero matrix D
  end

  sol = totm\P;

  if dmat==1
      // Find B and D
      sol_bd = matrix(sol,n+l,m);
      D = sol_bd(1:l,:);
      B = sol_bd(l+1:l+n,:);
  else
      // NEW zero matrix D
      B = matrix(sol,n,m);
      D = zeros(l,m);
      // end of zero matrix D
  end

end;


// **************************************
//               STEP 7 
// **************************************

if norm(res)>0.0000000001 then
  mydisp(sil,"      Computing ... System matrices G,L0 (Order "+string(n)+")");
  // Determine the residuals
    cov = res*res';  // Covariance [Q S;S' R]
  Qs = cov(1:n,1:n);Ss = cov(1:n,n+1:n+l);Rs = cov(n+1:n+l,n+1:n+l); 

  sig=lyap(A',-Qs','d');//WARNING!: transposition and negative Qs crucial to complay Matlab definition of dlyap..see help scilab and matlab
  G = A*sig*C' + Ss;
  L0 = C*sig*C' + Rs;

  // Determine K and Ro
  mydisp(sil,"      Computing ... Riccati solution")
  [K,Ro] = gl2kr(A,G,C,L0);
else
  Ro = [];
  K = [];
end;


//%%%%%%%%%%%%%%%%%%%%%%% END ALGORITHM%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if lhs > 6 then
  AUX = zeros((4*l+2*m)*i+1,2*(m+l)*i);
  if isempty(Uaux) then
      Uaux = 0;
  end
  info = [ds_flag,i,Uaux,y(1,1),Waux]; // in/out - i - u(1,1) - y(1,1) - W
  AUX(1,1:5) = info;
  bb = 1;
  AUX(bb+1:bb+2*(m+l)*i,1:2*(m+l)*i) = R;
  bb = bb+2*(m+l)*i;
  AUX(bb+1:bb+l*i,1:2*(m+l)*i) = Ob;
  bb = bb+l*i;
  AUX(bb+1:bb+l*i,1:l*i) = U;
  AUX(bb+1:bb+l*i,l*i+1) = ss;
end

m = idmodel();
m.A = A;
m.B = B;
m.C = C;
m.D = D;
m.K = K;
m.algorithm = 'Combined 4SID';

endfunction

