function [m] = substruc(y, u, i, na, nb, nc, nk, method)
//Estimate ARMAX model / A,C,B,C,D,K matrices of the system.
//Calling Sequence
//   [m] = substruc(y, u, i, na, nb, nc, nk)
// 
//Parameters
//   Inputs:
//     y: vector of outputs
//     u: vector of inputs
//     i: prediction horizon
//     na: number of poles = [number of coefficients of a(d)] - 1
//     nb: number of zeros + 1 = [number of coefficients of b(d)] - nk
//     nc: number of coefficients of the noise filter numerator - 1
//     nk: input-output time delay
//     method: dmat = 1 for nonzero matrix D, dmat = 0 for zero matrix D
//   Outputs:
//     m: idmodel structure (see help idmodel)
//
//Examples
//data = read('../demos/dryer.dat',-1,2); //data = [u y]
//bound = 400;
//y = data(1:bound,2); u = data(1:bound,1); //identification data
//yv = data(bound+1:$,2); uv = data(bound+1:$,1); //validation data
//na = 3; nb = 3; nc = 2; nk = 2; i=10;
//m = substruc(y,u,i,na,nb,nc,nk);
//[m.fit, yp] = valPred(m.a, m.b, m.c, yv, uv);
//plot(yv);
//plot(yp,'-r');
//
// See also
//  subspaceIdent
//  arx
//  rarx
//  arIdent
//  valPred
//  valSim
// 
//Bibliography
//  Overschee, DeMoor: Subspace identification for linear systems
//  Lyzell, Ch.: Initialization Methods for System Identification
//  Lyzell, Ch.: Handling Certain Structure Information in Subspace Identification

[lhs,rhs]=argn();
if (rhs < 7) then
    error('substruc needs at least seven arguments');
end

if nb > na | nc > na
    disp('It should be better na >= nb and na>=nc');
end

dmat = 0;
sizU = min(size(u)); // number of inputs
n = max(na,nb);

a = []; b = []; c = [];
Ubeta = []; Ybeta = []; Ualpha = []; Yalpha = []; ii = [];
iend = size(u,1)-2*i;

for ii = 1:iend
    j=ii+i;
    Ubeta =  [Ubeta  u(ii:ii+i-1,:,:)];
    Ybeta =  [Ybeta  y(ii:ii+i-1,1,:)];
    Ualpha = [Ualpha u(j:j+i-1,:,:)];
    Yalpha = [Yalpha y(j:j+i-1,1,:)];
end

// Estimate the extended observability matrix Obsv

pibeta = eye(iend,iend) - Ubeta'*pinv(Ubeta*Ubeta')*Ubeta;
Z = [Ualpha; Yalpha];
tosvd = Ybeta*pibeta*Z';
[U,S,V] = svd(tosvd);
U1 = U(:,1:n);
Obsv = U1;
gamai = pinv(U1);
Obsv2 = U1(2:$,:);


// Estimate A,C

[dummy,llu] = size(u);
[nny,lly] = size(y);
tosqrt = nny-2*i+1;
Y = blockhank(y/sqrt(tosqrt),2*i,tosqrt);
U = blockhank(u/sqrt(tosqrt),2*i,tosqrt);
R = triu(qrhouse([U;Y]'))';

A = pinv(Obsv(1:$-1,:))*Obsv(2:$,:);
a = mtlb_fliplr(coeff(poly(A,'lam'))); // characteristic equation
a(na+2:$) = 0;
A = [-a(2:$)', eye(n,n-1)];
C = eye(1,n);

Obsv = C;
for k = 2:i
  Obsv((k-1)*lly+1:k*lly,:) = Obsv((k-2)*lly+1:(k-1)*lly,:)*A;
end;
gama = Obsv(1:lly*(i-1),:);
gamai = pinv(Obsv);


// Estimate B,D

Right = [gamai*R((2*llu+lly)*i+1:2*(llu+lly)*i,1:(2*llu+lly)*i),zeros(n,lly) ; R(llu*i+1:2*llu*i,1:(2*llu+lly)*i+lly)];
Left = [pinv(gama)*R((2*llu+lly)*i+lly+1:2*(llu+lly)*i,1:(2*llu+lly)*i+lly); R((2*llu+lly)*i+1:(2*llu+lly)*i+lly,1:(2*llu+lly)*i+lly)];
B = []; D = [];
P = Left - [A;C]*Right(1:n,:); // page 125
P = P(:,1:2*llu*i);
Q = R(llu*i+1:2*llu*i,1:2*llu*i); // page 125
L1 = A * gamai;
L2 = C * gamai;
M  = [zeros(n,lly),pinv(gama)];
X  = [eye(lly,lly),zeros(lly,n);zeros(lly*(i-1),lly),gama];

sol = 0;
for k=1:i
    N = [[M(:,(k-1)*lly+1:lly*i)-L1(:,(k-1)*lly+1:lly*i),zeros(n,(k-1)*lly)];[-L2(:,(k-1)*lly+1:lly*i),zeros(lly,(k-1)*lly)]];
    if k==1
        N(n+1:n+lly,1:lly) = eye(lly,lly) + N(n+1:n+lly,1:lly);
    end
    N = N*X;
    sol = sol + kron(Q((k-1)*llu+1:k*llu,:)',N);
end

P = P(:);
solnew = [];
for k = 1:llu
    solnew = [solnew,sol(:,k*(lly+n)-n+1:k*(lly+n))];
end
sol = solnew;

// Determine the coeficients:
nu = ceil(nb/(na+1)); // number of virtual inputs
np = nu*na + nu; // total nuber of parameters in B and D matrix
nl = np - nb - 1; // number of parameters fixed by a linear constraint
// Linear equality constraints
constr = [zeros(nl, nu*na - nl) eye(nl,nl)];
Leq = kron(eye(sizU,sizU),constr); // constraints on B
Req = [zeros(sizU*nl,1)];

H=2*sol'*sol;
f=-2*sol'*P;
sol = qpsolve(H,f,Leq,Req,[],[],size(Leq,2));

B = matrix(sol,n,llu);
D = zeros(lly,llu);

// Estimate K
Zbeta = [Ubeta; Z];
pizbeta = Zbeta'*pinv(Zbeta*Zbeta')*Zbeta;
phialpha = kron(eye(i,i),D);
ObsvB = Obsv([1:$],:)*B;
for kk=2:size(phialpha,1)
    phialpha([kk:$],[kk-1]) = ObsvB([1:$-kk+1],:);
end
X = gamai*(Ybeta*pizbeta - phialpha*Ubeta); // states

betaa = i/2;
res = ([X(:,betaa+1:$); Ybeta(1,betaa:$-1)]) - ([A B; C D]*[X(:,betaa:$-1); Ubeta(1,betaa:$-1)]);

W = res(1:n,:);
V = res(n+1:$,:);

nu = ceil(nc/(na+1)); // number of virtual inputs
nac = n - nc;
constr = [zeros(nac, nc) eye(nac,nac)];
Leq = kron(eye(sizU,sizU),constr); // constraints on K
Req = [zeros(sizU*nac,1)];

Left = pinv(W);
Right = pinv(V);
H=2*Left'*Left;
f=-2*Left'*Right;
K = qpsolve(H,f,Leq,Req,[],[],size(Leq,2));

m = idmodel();
m.A = A;
m.B = B;
m.C = C;
m.D = D;
m.K = K;
// ARMAX polynomials
a = []; b = []; c = [];
m.a = [a; 1 -A(:,1)'];
m.b = [zeros(sizU,nk) B(1:nb,:)'];
m.c = [c; 1 K' + a(2:$)];

endfunction