function M = nng(FI, theta, A, b, na, nb, nc, nk, yi)
// Nonnegative Garrote (NNG) method for reducing the order of input vector theta.
//
// Calling Sequence
// [M] = nng(FI, theta, A, b, na, nb, nc, nk, yi)
// Parameters
//   Inputs:
// FI: matrix of regressors
// theta: vector of parameters
// A,b: inequality constraints for nng problem
// na: number of poles (length of polynom A of ARX model in theta)
// nb: number of zeros + 1 (length of polynom B of ARX model in theta)
// nc: number of zeros + 1 (length of polynom C of ARMAX model in theta)
// nk: time delay from input to output
// yi: vector of outputs
//   Outputs:
// M: structure with the modified parameters of theta
//       M.lambda:   parameter of model complexity
//       M.w:        table of weighting vectors of the original parameters
//       M.thetaNew: table of the new parameters
//       M.A:        table of the new coeficients of the polynomial A (ARX)
//       M.B:        table of the new coeficients of the polynomial B (ARX)
//       M.C:        table of the new coeficients of the polynomial C (ARMAX)
// Description
// Reducing the order of LTI model by modification (shrinkage) coefficients of LTI model arranged in theta using the modified Nonnegative Garrote (NNG) algorithm.
// 
// Examples
// see selstruc
// 
// See Also
//  selstruc
// 
// Authors
// 

if nc == [] // ARX
    
Y = yi(max(na+1,nb+nk):$,:);
THETA = diag(theta(2:$));
Q = 2*THETA*(FI)'*FI*THETA;
//f = -2*THETA*FI'*Y;

//------------Algorithm STEP1 - Initialization---------------
lambda = 0;
w = ones(size(THETA,1),1);
mu = zeros(size(THETA,1)+2,1); // wector of Lagrangian multipliers
ja = [ones(1,na) 0 ones(1,nb) 0]'; // set of active constraints (for mu)
muja = mu.*ja;
count = 1;
M.lambda(count) = lambda; // initial solution
M.w(:,count) = w; // initial solution
M.thetaNew(:,count) = theta;
M.A(:,count) = theta(1:na+1);
M.B(:,count) = theta(na+2:1+na+nb);

limit = 1e+4*%eps;

disp('      Reducing the order of the ARX model ... Please wait until the display shows: DONE.');
//waitId=waitbar('Order reducing...please wait...');
//realtimeinit(0.3);

steps = 300; // nuber of steps for waitbar
//------------------Algorithm STEP2---------------------
while lambda ~= %inf & count < steps
   // waitbar(3*count/steps,waitId);
    count = count + 1;
    Aja = A; bja = b;
    for kk=1:size(A,1)
        if ja(kk)==1
            Aja(kk,:) = A(kk,:);
            bja(kk) = b(kk);
        else
            Aja(kk,:) = 0;
            bja(kk) = 0;
        end
    end
    siz = size(A);
    Left = [Q Aja'; Aja zeros(siz(1),siz(1))];
    Right = [-1*ones(siz(2),1); zeros(siz(1),1)];
    //derivs = Left\Right;
    derivs = pinv(Left)*Right; // sloution of eq.(15)
    parW = derivs(1:siz(2));
    parMu = derivs(siz(2)+1:$);

//------------------Algorithm STEP3---------------------
    DeltaLambda.value = %nan*ones(length(ja),1);
    DeltaLambda.property = %nan*ones(length(ja),1);
    stop = 0;
    for ll=1:length(ja)
        if ja(ll) == 0
            deltaLambda = (b(ll) - A(ll,:)*w)/(A(ll,:)*parW);
            if A(ll,:)*parW > 0 & deltaLambda >= 0
                DeltaLambda.value(ll) = deltaLambda;
                DeltaLambda.property(ll) = 1; // to add to j
                stop = 1;
            end
        end
        if ja(ll) == 1
            deltaLambda = (-muja(ll))/parMu(ll);
            if parMu(ll) < 0 & deltaLambda >= 0
                DeltaLambda.value(ll) = deltaLambda;
                DeltaLambda.property(ll) = 0; // to remove from j
                stop = 1;
            end
        end
    end
    
    if stop == 0
        deltaLambda = %inf; // set termination criterion
    else
        // update active constraints j:
        deltaLambda = min(DeltaLambda.value);
        for pp=1:length(DeltaLambda.value)
            if DeltaLambda.value(pp) == deltaLambda
                if DeltaLambda.property(pp) == 1
                    ja(pp) = 1; //disp('Added.');
                end
                if DeltaLambda.property(pp) == 0
                    ja(pp) = 0; //disp('Removed.');
                end
            end
        end
    end

//------------------Algorithm STEP4---------------------
    lambda = lambda + deltaLambda;
    w = w + parW*deltaLambda;
    for oo=1:length(w) // zeroing the insignificant coefficients of w
        if abs(w(oo)) <= limit
            w(oo) = 0;
        end
    end
    muja = muja + (parMu.*ja)*deltaLambda;
    M.lambda(count) = lambda; // new solution
    M.w(:,count) = w; // new solution
    ws = THETA*w;
    M.thetaNew(:,count) = [1; ws];
    M.A(:,count)  = [1; ws(1:na)];
    M.B(:,count) = ws(na+1:$);
 
end
//------------------------END---------------------------

//winclose(waitId);

end // end if ARX
/////////////////////////////////////////////////////////////////////////////////////////

if nc ~= [] // ARMAX
    
    Y = yi(max(na+1,nb+nk):$,:);
    Q = 2*THETA*(FI)'*FI*THETA;
    //f = -2*THETA*FI'*Y;

//------------Algorithm STEP1 - Initialization---------------
lambda = 0;
w = ones(size(THETA,1),1);
mu = zeros(size(THETA,1)+3,1); // wector of Lagrangian multipliers
ja = [ones(1,na) 0 ones(1,nb) 0 ones(1,nc) 0]'; // set of active constraints (for mu)
muja = mu.*ja;
count = 1;
M.lambda(count) = lambda; // initial solution
M.w(:,count) = w; // initial solution
M.thetaNew(:,count) = theta;
M.A(:,count) = theta(1:na+1);
M.B(:,count) = theta(na+2:1+na+nb);
M.C(:,count) = theta(2+na+nb:$);

limit = 1e+4*%eps;

disp('      Reducing the order of the ARMAX model ... Please wait until the display shows: DONE.');
//waitId=waitbar('Order reducing...please wait...');
//realtimeinit(0.3);

steps = 300; // nuber of steps for waitbar
//------------------Algorithm STEP2---------------------
while lambda ~= %inf & count < steps
    //waitbar(3*count/steps,waitId);
    count = count + 1;
    Aja = A; bja = b;
    for kk=1:size(A,1)
        if ja(kk)==1
            Aja(kk,:) = A(kk,:);
            bja(kk) = b(kk);
        else
            Aja(kk,:) = 0;
            bja(kk) = 0;
        end
    end
    siz = size(A);
    Left = [Q Aja'; Aja zeros(siz(1),siz(1))];
    Right = [-1*ones(siz(2),1); zeros(siz(1),1)];
    //derivs = Left\Right;
    derivs = pinv(Left)*Right; // sloution of eq.(15)
    parW = derivs(1:siz(2));
    parMu = derivs(siz(2)+1:$);

//------------------Algorithm STEP3---------------------
    DeltaLambda.value = %nan*ones(length(ja),1);
    DeltaLambda.property = %nan*ones(length(ja),1);
    stop = 0;
    for ll=1:length(ja)
        if ja(ll) == 0
            deltaLambda = (b(ll) - A(ll,:)*w)/(A(ll,:)*parW);
            if A(ll,:)*parW > 0 & deltaLambda >= 0
                DeltaLambda.value(ll) = deltaLambda;
                DeltaLambda.property(ll) = 1; // to add to j
                stop = 1;
            end
        end
        if ja(ll) == 1
            deltaLambda = (-muja(ll))/parMu(ll);
            if parMu(ll) < 0 & deltaLambda >= 0
                DeltaLambda.value(ll) = deltaLambda;
                DeltaLambda.property(ll) = 0; // to remove from j
                stop = 1;
            end
        end
    end
    
    if stop == 0
        deltaLambda = %inf; // set termination criterion
    else
        // update active constraints j:
        deltaLambda = min(DeltaLambda.value);
        for pp=1:length(DeltaLambda.value)
            if DeltaLambda.value(pp) == deltaLambda
                if DeltaLambda.property(pp) == 1
                    ja(pp) = 1; //disp('Added.');
                end
                if DeltaLambda.property(pp) == 0
                    ja(pp) = 0; //disp('Removed.');
                end
            end
        end
    end

//------------------Algorithm STEP4---------------------
    lambda = lambda + deltaLambda;
    w = w + parW*deltaLambda;
    for oo=1:length(w) // zeroing the insignificant coefficients of w
        if abs(w(oo)) <= limit
            w(oo) = 0;
        end
    end
    muja = muja + (parMu.*ja)*deltaLambda;
    M.lambda(count) = lambda; // new solution
    M.w(:,count) = w; // new solution
    ws = THETA*w;
    M.A(:,count)  = [1; ws(1:na)];
    M.B(:,count) = ws(na+1:na+nb);
    M.C(:,count) = [1; ws(na+nb+1:$)];
    M.thetaNew(:,count) = [M.A(:,count); M.B(:,count); M.C(:,count)];
 
end
//------------------------END---------------------------

//winclose(waitId);

end // end if ARMAX

disp('DONE.');

endfunction