function [x,X] = rtls(A,b,x0,delta,tau,L)
//Regularized Total-Least Squares algorithm to a set of linear equations (A + E)x = b + f where E and f are white noise: *** min||E,f||_F subject to (A + E)x = b + f ||Lx||<=delta ***
//
// Calling Sequence
// [x] = rtls(A, b) // without priori estimate of x
// [x] = rtls(A, b, x0) // define priori estimate of x
// [x] = rtls(A, b, x0, delta) // define tunning parameter delta
// [x] = rtls(A, b, x0, delta, tau) // define tunning parameters delta, tau
// [x] = rtls(A, b, x0, delta, tau, L) // define tunning parameters delta, tau, L
// Parameters
// A: A
// b: b
// x0: initial estimate of x
// delta: tunning parameter (default delta=1)
// tau: tunning parameter (default tau = 5)
// L: weighting matrix - tunning parameter (default L=0.1*eye(size(A,2),size(A,2)))
// x: Regularized Total-Least Squares solution of (A + E)x = b + f
// 
// Description
// Regularized Total-Least Squares solution to a set of linear 
// equations (A + E)x = b + f where E and f are white noise. 
// This function is used to estimate parameters of so called
// Errors-In-Variables systems (both measured input (A+E) and
// output (b+f) data are corrupted by white noise).
// The R-TLS solution minimizes the
// *** min||E,f||_F subject to (A + E)x = b + f ||Lx||<=delta ***
// where delta, tau, L are tunning variables described above.
// 
// Examples
// A = [1 3 2 4; 3 2 3 2; 3 4 3 1; -9 -1 2 -1];
// b = [32 21 22 -12]';
// [x] = rtls(A,b)
// x0 = A\b; compute initial estimate
// [x] = rtls(A,b,x0); 
// A*x
// 
// See Also
//  tls
// 
// Authors
// 
// Bibliography
//     Hongbin Guo, Rosemary A. Renaut: A regularized total least squares algorithm

[LHS RHS] = argn(0);
if (RHS < 2 | RHS > 6)
    error('Not enough input arguments. Use e.g. x = rtls(A,b).');
end
if (size(A,1) ~= size(b,1))
    error('size(A,1) and size(b,1) must be the same.')
end

n = size(A,2);

if (RHS == 2 )
    x0 = A \ b;
    L = 1*eye(size(A,2),size(A,2));
    delta = 1;
    tau = 5;
end
if (RHS == 3 )
    L = 1*eye(size(A,2),size(A,2));
    delta = 1;
    tau = 5;
end
if (RHS == 4 )
    L = 1*eye(size(A,2),size(A,2));
    tau = 5;
end
if (RHS == 5 )
    L = 1*eye(size(A,2),size(A,2));
end

x = x0;
X = [];

//-------------Regularized Total-Least Squares algorithm-------------
norma = norm(A*x - b);
lambdai = -norma/(1+norm(x));
mu = (-1/delta^2)*(((b'*(A*x - b))/(1+norm(x))) + (norma/(1+norm(x))^2));
lambdal = mu*(1 + norm(x));
steps = 0;
ro = 0;
while abs(ro/lambdai) < tau
    z = [x' -1]/(sqrt(1+norm(x)));
    B = [A'*A + lambdal*L'*L A'*b; b'*A, -lambdal*delta^2 + b'*b];
    Lhs = B + lambdai*eye(size(lambdai,1));
    Rhs = z;
    sol = Rhs/Lhs;
    if abs(sol(n+1)) > %eps
        x = [-sol(1:n)/sol(n+1)]';
        lambdai = norma/(1+(norm(x))^2);
        mu = (-1/delta^2)*((b'*(A*x - b))/(1+(norm(x))^2)) + (norma/((1+(norm(x))^2))^2);
        lambdal = mu*(1 + norm(x));
        z = sol/norm(sol);
        ro = mu*(norm(L*x) - delta^2);
        if abs(ro/lambdai) < tau | steps > 1000
            disp('rtls.sci: |Breakpoint ro/lambdai < tau| REACHED');
            break;
        end
    else
        sol = sol/norm(sol);
        z = (z + sol)/norm(z + sol);  
    end
    steps = steps + 1;
    X(:,steps) = x;
end

endfunction
